=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-02-21 22:41:44 +0000
+++ bzrlib/builtins.py	2006-02-21 22:48:58 +0000
@@ -35,10 +35,12 @@
 from bzrlib.log import show_one_log
 from bzrlib.merge import Merge3Merger
 from bzrlib.option import Option
+from bzrlib.progress import DummyProgress
 from bzrlib.revisionspec import RevisionSpec
 import bzrlib.trace
 from bzrlib.trace import mutter, note, log_error, warning, is_quiet
 from bzrlib.transport.local import LocalTransport
+import bzrlib.ui
 from bzrlib.workingtree import WorkingTree
 
 
@@ -1746,7 +1748,8 @@
         try:
             conflict_count = merge(other, base, check_clean=(not force),
                                    merge_type=merge_type, reprocess=reprocess,
-                                   show_base=show_base)
+                                   show_base=show_base, 
+                                   pb=bzrlib.ui.ui_factory.progress_bar())
             if conflict_count != 0:
                 return 1
             else:
@@ -1849,7 +1852,7 @@
         else:
             rev_id = revision[0].in_history(tree.branch).rev_id
         tree.revert(file_list, tree.branch.repository.revision_tree(rev_id),
-                    not no_backup)
+                    not no_backup, bzrlib.ui.ui_factory.progress_bar())
 
 
 class cmd_assert_fail(Command):
@@ -2156,7 +2159,8 @@
 def merge(other_revision, base_revision,
           check_clean=True, ignore_zero=False,
           this_dir=None, backup_files=False, merge_type=Merge3Merger,
-          file_list=None, show_base=False, reprocess=False):
+          file_list=None, show_base=False, reprocess=False, 
+          pb=DummyProgress()):
     """Merge changes into a tree.
 
     base_revision
@@ -2196,7 +2200,7 @@
                               " type. %s" % merge_type)
     if reprocess and show_base:
         raise BzrCommandError("Cannot reprocess and show base.")
-    merger = Merger(this_tree.branch, this_tree=this_tree)
+    merger = Merger(this_tree.branch, this_tree=this_tree, pb=pb)
     merger.check_basis(check_clean)
     merger.set_other(other_revision)
     merger.set_base(base_revision)

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2006-02-21 21:53:59 +0000
+++ bzrlib/merge.py	2006-02-21 22:48:58 +0000
@@ -37,6 +37,7 @@
 from bzrlib.merge3 import Merge3
 import bzrlib.osutils
 from bzrlib.osutils import rename, pathjoin
+from progress import DummyProgress
 from bzrlib.revision import common_ancestor, is_ancestor, NULL_REVISION
 from bzrlib.symbol_versioning import *
 from bzrlib.trace import mutter, warning, note
@@ -79,7 +80,8 @@
 
 
 class Merger(object):
-    def __init__(self, this_branch, other_tree=None, base_tree=None, this_tree=None):
+    def __init__(self, this_branch, other_tree=None, base_tree=None, 
+                 this_tree=None, pb=DummyProgress()):
         object.__init__(self)
         assert this_tree is not None, "this_tree is required"
         self.this_branch = this_branch
@@ -95,6 +97,7 @@
         self.interesting_ids = None
         self.show_base = False
         self.reprocess = False
+        self._pb = pb 
 
     def revision_tree(self, revision_id):
         return self.this_branch.repository.revision_tree(revision_id)
@@ -202,7 +205,8 @@
             try:
                 self.base_rev_id = common_ancestor(self.this_basis, 
                                                    self.other_basis, 
-                                                   self.this_branch.repository)
+                                                   self.this_branch.repository,
+                                                   self._pb)
             except NoCommonAncestor:
                 raise UnrelatedBranches()
             self.base_tree = _get_revid_tree(self.this_branch, self.base_rev_id,
@@ -236,7 +240,7 @@
         elif self.show_base:
             raise BzrError("Showing base is not supported for this"
                                   " merge type. %s" % self.merge_type)
-        merge = self.merge_type(**kwargs)
+        merge = self.merge_type(pb=self._pb, **kwargs)
         if len(merge.cooked_conflicts) == 0:
             if not self.ignore_zero:
                 note("All changes applied successfully.")
@@ -314,7 +318,7 @@
     history_based = False
 
     def __init__(self, working_tree, this_tree, base_tree, other_tree, 
-                 reprocess=False, show_base=False):
+                 reprocess=False, show_base=False, pb=DummyProgress()):
         """Initialize the merger object and perform the merge."""
         object.__init__(self)
         self.this_tree = working_tree
@@ -324,17 +328,20 @@
         self.cooked_conflicts = []
         self.reprocess = reprocess
         self.show_base = show_base
+        self.pb = pb
 
         all_ids = set(base_tree)
         all_ids.update(other_tree)
-        self.tt = TreeTransform(working_tree)
+        self.tt = TreeTransform(working_tree, self.pb)
         try:
-            for file_id in all_ids:
+            for num, file_id in enumerate(all_ids):
+                self.pb.update('Preparing file merge', num+1, len(all_ids))
                 self.merge_names(file_id)
                 file_status = self.merge_contents(file_id)
                 self.merge_executable(file_id, file_status)
+            self.pb.clear()
                 
-            fs_conflicts = resolve_conflicts(self.tt)
+            fs_conflicts = resolve_conflicts(self.tt, self.pb)
             self.cook_conflicts(fs_conflicts)
             for line in conflicts_strings(self.cooked_conflicts):
                 warning(line)
@@ -700,11 +707,12 @@
     supports_reprocess = False
     supports_show_base = False
 
-    def __init__(self, working_tree, this_tree, base_tree, other_tree):
+    def __init__(self, working_tree, this_tree, base_tree, other_tree, 
+                 pb=DummyProgress()):
         self.this_revision_tree = self._get_revision_tree(this_tree)
         self.other_revision_tree = self._get_revision_tree(other_tree)
         super(WeaveMerger, self).__init__(working_tree, this_tree, 
-                                          base_tree, other_tree)
+                                          base_tree, other_tree, pb=pb)
 
     def _get_revision_tree(self, tree):
         """Return a revision tree releated to this tree.
@@ -800,7 +808,8 @@
                 reprocess=False, 
                 other_rev_id=None,
                 interesting_files=None,
-                this_tree=None):
+                this_tree=None,
+                pb=DummyProgress()):
     """Primary interface for merging. 
 
         typical use is probably 
@@ -813,7 +822,8 @@
              DeprecationWarning,
              stacklevel=2)
         this_tree = this_branch.bzrdir.open_workingtree()
-    merger = Merger(this_branch, other_tree, base_tree, this_tree=this_tree)
+    merger = Merger(this_branch, other_tree, base_tree, this_tree=this_tree, 
+                    pb=pb)
     merger.backup_files = backup_files
     merger.merge_type = merge_type
     merger.interesting_ids = interesting_ids

=== modified file 'bzrlib/revision.py'
--- bzrlib/revision.py	2006-02-08 06:15:10 +0000
+++ bzrlib/revision.py	2006-02-16 18:52:17 +0000
@@ -20,6 +20,7 @@
 import bzrlib.errors
 from bzrlib.graph import node_distances, select_farthest, all_descendants
 from bzrlib.osutils import contains_whitespace
+from bzrlib.progress import DummyProgress
 
 NULL_REVISION="null:"
 
@@ -279,17 +280,24 @@
     return root, ancestors, descendants, common
 
 
-def common_ancestor(revision_a, revision_b, revision_source):
+def common_ancestor(revision_a, revision_b, revision_source, 
+                    pb=DummyProgress()):
     try:
-        root, ancestors, descendants, common = \
-            combined_graph(revision_a, revision_b, revision_source)
-    except bzrlib.errors.NoCommonRoot:
-        raise bzrlib.errors.NoCommonAncestor(revision_a, revision_b)
-        
-    distances = node_distances (descendants, ancestors, root)
-    farthest = select_farthest(distances, common)
-    if farthest is None or farthest == NULL_REVISION:
-        raise bzrlib.errors.NoCommonAncestor(revision_a, revision_b)
+        try:
+            pb.update('Picking ancestor', 1, 3)
+            root, ancestors, descendants, common = \
+                combined_graph(revision_a, revision_b, revision_source)
+        except bzrlib.errors.NoCommonRoot:
+            raise bzrlib.errors.NoCommonAncestor(revision_a, revision_b)
+            
+        pb.update('Picking ancestor', 2, 3)
+        distances = node_distances (descendants, ancestors, root)
+        pb.update('Picking ancestor', 3, 2)
+        farthest = select_farthest(distances, common)
+        if farthest is None or farthest == NULL_REVISION:
+            raise bzrlib.errors.NoCommonAncestor(revision_a, revision_b)
+    finally:
+        pb.clear()
     return farthest
 
 

=== modified file 'bzrlib/transform.py'
--- bzrlib/transform.py	2006-02-21 17:36:08 +0000
+++ bzrlib/transform.py	2006-02-21 21:20:21 +0000
@@ -23,6 +23,7 @@
                            ExistingLimbo, ImmortalLimbo)
 from bzrlib.inventory import InventoryEntry
 from bzrlib.osutils import file_kind, supports_executable, pathjoin
+from bzrlib.progress import DummyProgress
 from bzrlib.trace import mutter, warning
 
 
@@ -60,7 +61,7 @@
      * version_file
      * set_executability
     """
-    def __init__(self, tree):
+    def __init__(self, tree, pb=DummyProgress()):
         """Note: a write lock is taken on the tree.
         
         Use TreeTransform.finalize() to release the lock
@@ -94,6 +95,7 @@
         self._tree_id_paths = {}
         self._new_root = self.trans_id_tree_file_id(tree.get_root_id())
         self.__done = False
+        self._pb = pb
 
     def __get_root(self):
         return self._new_root
@@ -676,7 +678,9 @@
         """
         tree_paths = list(self._tree_path_ids.iteritems())
         tree_paths.sort(reverse=True)
-        for path, trans_id in tree_paths:
+        for num, data in enumerate(tree_paths):
+            path, trans_id = data
+            self._pb.update('removing file', num+1, len(tree_paths))
             full_path = self._tree.abspath(path)
             if trans_id in self._removed_contents:
                 self.delete_any(full_path)
@@ -697,6 +701,7 @@
                 if file_id is not None:
                     limbo_inv[trans_id] = inv[file_id]
                     del inv[file_id]
+        self._pb.clear()
 
     def _apply_insertions(self, inv, limbo_inv):
         """Perform tree operations that insert directory/inventory names.
@@ -705,7 +710,9 @@
         limbo any files that needed renaming.  This must be done in strict
         parent-to-child order.
         """
-        for path, trans_id in self.new_paths():
+        new_paths = self.new_paths()
+        for num, (path, trans_id) in enumerate(new_paths):
+            self._pb.update('adding file', num+1, len(new_paths))
             try:
                 kind = self._new_contents[trans_id]
             except KeyError:
@@ -736,6 +743,7 @@
             # requires files and inventory entries to be in place
             if trans_id in self._new_executability:
                 self._set_executability(path, inv, trans_id)
+        self._pb.clear()
 
     def _set_executability(self, path, inv, trans_id):
         """Set the executability of versioned files """
@@ -985,13 +993,14 @@
     return has_contents, contents_mod, meta_mod
 
 
-def revert(working_tree, target_tree, filenames, backups=False):
+def revert(working_tree, target_tree, filenames, backups=False, 
+           pb=DummyProgress()):
     """Revert a working tree's contents to those of a target tree."""
     interesting_ids = find_interesting(working_tree, target_tree, filenames)
     def interesting(file_id):
         return interesting_ids is None or file_id in interesting_ids
 
-    tt = TreeTransform(working_tree)
+    tt = TreeTransform(working_tree, pb)
     try:
         trans_id = {}
         def trans_id_file_id(file_id):
@@ -1000,9 +1009,10 @@
             except KeyError:
                 return tt.trans_id_tree_file_id(file_id)
 
-        for file_id in topology_sorted_ids(target_tree):
-            if not interesting(file_id):
-                continue
+        sorted_interesting = [i for i in topology_sorted_ids(target_tree) if
+                              interesting(i)]
+        for id_num, file_id in enumerate(sorted_interesting):
+            pb.update("Reverting file", id_num+1, len(sorted_interesting))
             if file_id not in working_tree.inventory:
                 entry = target_tree.inventory[file_id]
                 parent_id = trans_id_file_id(entry.parent_id)
@@ -1011,28 +1021,33 @@
             else:
                 change_entry(tt, file_id, working_tree, target_tree, 
                              trans_id_file_id, backups, trans_id)
-        for file_id in working_tree.inventory:
-            if not interesting(file_id):
-                continue
+        wt_interesting = [i for i in working_tree.inventory if interesting(i)]
+        for id_num, file_id in enumerate(wt_interesting):
+            pb.update("New file check", id_num+1, len(sorted_interesting))
             if file_id not in target_tree:
                 tt.unversion_file(tt.trans_id_tree_file_id(file_id))
-        raw_conflicts = resolve_conflicts(tt)
+        raw_conflicts = resolve_conflicts(tt, pb)
         for line in conflicts_strings(cook_conflicts(raw_conflicts, tt)):
             warning(line)
         tt.apply()
     finally:
         tt.finalize()
-
-
-def resolve_conflicts(tt):
+        pb.clear()
+
+
+def resolve_conflicts(tt, pb=DummyProgress()):
     """Make many conflict-resolution attempts, but die if they fail"""
     new_conflicts = set()
-    for n in range(10):
-        conflicts = tt.find_conflicts()
-        if len(conflicts) == 0:
-            return new_conflicts
-        new_conflicts.update(conflict_pass(tt, conflicts))
-    raise MalformedTransform(conflicts=conflicts)
+    try:
+        for n in range(10):
+            pb.update('Resolution pass', n+1, 10)
+            conflicts = tt.find_conflicts()
+            if len(conflicts) == 0:
+                return new_conflicts
+            new_conflicts.update(conflict_pass(tt, conflicts))
+        raise MalformedTransform(conflicts=conflicts)
+    finally:
+        pb.clear()
 
 
 def conflict_pass(tt, conflicts):

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-02-21 01:05:33 +0000
+++ bzrlib/workingtree.py	2006-02-21 21:06:53 +0000
@@ -81,6 +81,7 @@
                             rename,
                             supports_executable,
                             )
+from bzrlib.progress import DummyProgress
 from bzrlib.revision import NULL_REVISION
 from bzrlib.symbol_versioning import *
 from bzrlib.textui import show_status
@@ -89,6 +90,7 @@
 from bzrlib.transform import build_tree
 from bzrlib.transport import get_transport
 from bzrlib.transport.local import LocalTransport
+import bzrlib.ui
 import bzrlib.xml5
 
 
@@ -849,7 +851,8 @@
                 merge_inner(self.branch,
                             self.basis_tree(), 
                             repository.revision_tree(other_revision),
-                            this_tree=self)
+                            this_tree=self, 
+                            pb=bzrlib.ui.ui_factory.progress_bar())
                 self.set_last_revision(self.branch.last_revision())
             return count
         finally:
@@ -1077,11 +1080,12 @@
         self._write_inventory(inv)
 
     @needs_write_lock
-    def revert(self, filenames, old_tree=None, backups=True):
+    def revert(self, filenames, old_tree=None, backups=True, 
+               pb=DummyProgress()):
         from transform import revert
         if old_tree is None:
             old_tree = self.basis_tree()
-        revert(self, old_tree, filenames, backups)
+        revert(self, old_tree, filenames, backups, pb)
         if not len(filenames):
             self.set_pending_merges([])
 

